"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PromiseState;
(function (PromiseState) {
    PromiseState["PENDING"] = "pending";
    PromiseState["FULFILLED"] = "fulfilled";
    PromiseState["REJECTED"] = "rejected";
})(PromiseState || (PromiseState = {}));
function isPromise(val) {
    return val && typeof val.then === 'function';
}
var Promise = /** @class */ (function () {
    function Promise(executor) {
        this._state = PromiseState.PENDING;
        this._resolve = this._resolve.bind(this);
        this._reject = this._reject.bind(this);
        this._onFulfilled = [];
        this._onRejected = [];
        this._onFinally = [];
        try {
            executor(this._resolve, this._reject);
        }
        catch (e) {
            this._reject(e);
        }
    }
    Promise.prototype._resolve = function (val) {
        if (this._state !== PromiseState.PENDING) {
            return;
        }
        this._state = PromiseState.FULFILLED;
        this._value = val;
        this._onFulfilled.forEach(function (onFulfill) {
            onFulfill(val);
        });
        this._finally();
    };
    Promise.prototype._reject = function (val) {
        if (this._state !== PromiseState.PENDING) {
            return;
        }
        this._state = PromiseState.REJECTED;
        this._value = val;
        this._onRejected.forEach(function (onRejected) {
            onRejected(val);
        });
        this._finally();
    };
    Promise.prototype._finally = function () {
        this._onFinally.forEach(function (onFinally) { return onFinally(); });
    };
    // handles `then` and `catch` calls when the promise is still pending resolution
    Promise.prototype._handlePending = function (resolve, reject, onFulfilled, onRejected) {
        if (this._state !== PromiseState.PENDING) {
            return;
        }
        if (onFulfilled) {
            this._onFulfilled.push(function (val) {
                var retVal = onFulfilled(val);
                if (isPromise(retVal)) {
                    return retVal.then(resolve, resolve);
                }
                resolve(retVal);
            });
        }
        else {
            this._onFulfilled.push(resolve);
        }
        if (onRejected) {
            this._onRejected.push(function (val) {
                var retVal = onRejected(val);
                if (isPromise(retVal)) {
                    return retVal.then(resolve, resolve);
                }
                resolve(retVal);
            });
        }
        else {
            this._onRejected.push(reject);
        }
    };
    // handles when `then` or `catch` is called when the promise has already
    // been resolved or rejected
    Promise.prototype._handleResolvedRejected = function (resolve, reject, onFulfilled, onRejected) {
        if (this._state === PromiseState.PENDING) {
            return;
        }
        if (this._state === PromiseState.FULFILLED) {
            if (onFulfilled) {
                var retVal = onFulfilled(this._value);
                if (isPromise(retVal)) {
                    return retVal.then(resolve, resolve);
                }
                return resolve(retVal);
            }
            return resolve(this._value);
        }
        if (onRejected) {
            var retVal = onRejected(this._value);
            if (isPromise(retVal)) {
                return retVal.then(resolve, resolve);
            }
            return resolve(retVal);
        }
        return reject(this._value);
    };
    Promise.prototype.then = function (onFulfilled, onRejected) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this._handlePending(resolve, reject, onFulfilled, onRejected);
            _this._handleResolvedRejected(resolve, reject, onFulfilled, onRejected);
        });
    };
    Promise.prototype.catch = function (onRejected) {
        return this.then(undefined, onRejected);
    };
    Promise.prototype.finally = function (onFinally) {
        var _this = this;
        return new Promise(function (resolve) {
            _this._onFinally.push(function () {
                onFinally();
                resolve();
            });
        });
    };
    Promise.all = function (promises) {
        if (promises === void 0) { promises = []; }
        return new Promise(function (resolve, reject) {
            var retVal = [];
            var total = promises.length;
            var counter = 0;
            var success = function (val, index) {
                retVal[index] = val;
                counter++;
                if (counter === total) {
                    resolve(retVal);
                }
            };
            promises.forEach(function (p, index) {
                p.then(function (val) { return success(val, index); }, reject);
            });
        });
    };
    Promise.race = function (promises) {
        if (promises === void 0) { promises = []; }
        return new Promise(function (resolve, reject) {
            promises.forEach(function (p, index) {
                p.then(resolve, reject);
            });
        });
    };
    Promise.resolve = function (val) {
        return new Promise(function (resolve) { return resolve(val); });
    };
    Promise.reject = function (val) {
        return new Promise(function (_, reject) { return reject(val); });
    };
    return Promise;
}());
Object.defineProperty(Promise, 'length', {
    configurable: false,
    enumerable: false,
    value: 1,
});
exports.default = Promise;
function polyfill(w) {
    if (w === void 0) { w = window; }
    w.Promise = Promise;
}
exports.polyfill = polyfill;
